/*
 *              calib.cpp
 *
 *              Module to calibrate and test the Hall Effect sensor
 *              for the ME 161 Haptic Display
 *
 *              -Allison Okamura 11/12/96
 *              -Allison Okamura 11/17/98
 */

/* Includes */
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <iostream.h>
#include <math.h>
#include <graphics.h>
#include "ioboard.h"
#include "display.h"
#include "haptic.h"
#include "readcfg.h"

/* Defines */
#define HI       1
#define LO                0

/* Global Variables */

/* Functions */

/*--------------------- calibrate --------------------------*
 *  Full calibration Routine                                *
 *----------------------------------------------------------*/
void calibrate(void)
{
	int error = LO, done = LO;
	char filename[10], mfilename[15], key;
	//char station[2];
	FILE *fp;
	long int sensor;
	char angle[10];
	int index = 1, station_num = 0;

	// Prompt for group number
	clrscr();
   textcolor(CYAN);
   gotoxy(1,1); cprintf("Your file name must ...be eight characters or less");
   gotoxy(1,2); cprintf("                    ...begin with a letter");
	gotoxy(1,3); cprintf("                    ...not have an extension");
	gotoxy(1,4); cprintf("Use a different file name for each calibration.");
	textcolor(WHITE);
	gotoxy(1,6); cprintf("Enter a file name: ");
	gets(filename);

	// Open file
	sprintf(mfilename, "%s.m", filename);
	fp = fopen(mfilename, "w");
	textcolor(CYAN);
   if (fp == NULL) {
	gotoxy(1,8); cprintf("Error opening file.");
      gotoxy(1,9); cprintf("Check that you have followed the instructions above.");
		gotoxy(1,10); cprintf("Hit any key to return to main menu");
      while (!kbhit());
		getch();
      error = HI;
	} else {
		gotoxy(1,8); cprintf ("Your matlab file will be called %s. \n\n", mfilename);
	}

	// get station number
	if (!error) {
		textcolor(WHITE);

		station_num = 1;
		/*gotoxy(1,10); cprintf("Enter your station number (1-4): ");
		gets(station);
		sscanf(station, "%d", &station_num);
		station_num = 1;
		if ((station_num > 4) || (station_num < 1)) {
		textcolor(CYAN);
			gotoxy(1,12); cprintf("You have entered an incorrect station number.");
			gotoxy(1,13); cprintf("Hit any key to return to main menu");
			while (!kbhit());
			getch();
			error = HI;
		}
		*/
	}

	// choose ad channel
	init_board();
	scan_channels((station_num-1),(station_num-1));

	if (!error) {
		gotoxy(1,16); cprintf("You are now ready to start taking data\n");
		gotoxy(1,17); cprintf("Hit any key to continue");
		while (!kbhit()) ;
		getch();
		clrscr();

		// Taking data
		textcolor(CYAN);
		gotoxy(1,1); cprintf("Calibration Instructions:\n");
		gotoxy(1,2); cprintf("You must be connected to Station 1\n");
		gotoxy(1,3); cprintf("Hit any key (execpt 'q') to store the current sensor reading.\n");
		gotoxy(1,4); cprintf("Then type the measured angle (between 230 and 310 degrees) and hit enter.\n");
		gotoxy(1,5); cprintf("This program will allow you to take eight data points.\n");
		gotoxy(1,6); cprintf("Hit q to quit anytime.");

		textcolor(GREEN);
		gotoxy( 1,8); cprintf("#");
		gotoxy( 5,8); cprintf("Reading from sensor");
		gotoxy(30,8); cprintf("Measured Angle (deg)");

		textcolor(WHITE);
		while (!done && !error && (index < 9)) {
			gotoxy(1,9+index); cprintf("%d", index);
			sensor = read_ad();
			gotoxy(5,9+index); cprintf("%ld  ", sensor);
			if (kbhit()) {
				key = getch();
				if (key == 'q') done = HI;
					else {
						// read angle
						gotoxy(30,9+index);
			 gets(angle);

			 // print to file
			 fprintf(fp, "sensor(%d) = %ld; angle(%d) = %s;\n", index, sensor, index, angle);

			 // increment index
					index++;
		 }
			}
		}
	fclose(fp);

		gotoxy(1,20);
		textcolor(CYAN);
		if (index < 8) {
	cprintf("You did not take enough data points.");
	 gotoxy(1,21); cprintf("Run the calibration again");
		} else {
			cprintf("Datafile completed.");
	 gotoxy(1,21); cprintf("After you have taken 3 sets of data, go to Matlab.");
		}
		gotoxy(1,22); printf("Hit any key to return to main menu");
		while (!kbhit());
		getch();
	}
}

/*---------------------- test_cal --------------------------*
 *  Test calibration                                        *
 *----------------------------------------------------------*/
void test_cal(void)
{
	int done = LO, station_num, group_num;
	char string[4], station[2];
	char astring[4],  bstring[4], cstring[4], dstring[4];
	float sensor, angle, position, last_position;
	float a, b, c, d;

	// Prompt for coefficients
	clrscr();
	textcolor(CYAN);
	gotoxy(1,1); cprintf("Calibration Test");
	gotoxy(1,3); cprintf("Enter your group number: ");

	textcolor(WHITE);
	gets(string);
	sscanf(string, "%i", &group_num);

	// Read calibration values from data file
	sprintf(astring, "A%i", group_num);
	sprintf(bstring, "B%i", group_num);
	sprintf(cstring, "C%i", group_num);
	sprintf(dstring, "D%i", group_num);
	ReadCfgAddFloat(astring,&a,0);
	ReadCfgAddFloat(bstring,&b,0);
	ReadCfgAddFloat(cstring,&c,0);
	ReadCfgAddFloat(dstring,&d,0);
	ReadCfgExecute("calib.dat",RCE_APPLY_DEFAULTS);
	ReadCfgClear();				// clear config variable list

	// Example haptic paddle (using allison*.m)
	//a = -1.119191291975966e-008;
	//b = 6.927794830773996e-005;
	//c = -0.16055306378742;
	//d = 3.552250578841232e+002;

	gotoxy(1,11); cprintf("Enter in the station number (1-2): ");
	gotoxy(36,11); gets(station);
	sscanf(station, "%d", &station_num);

	// read only one ad channel
	scan_channels((station_num-1),(station_num-1));

	clrscr();
	init_graphics();

	gotoxy(1,1); printf("Hit any key to quit");
	gotoxy(1,3); printf("sensor       :");
	gotoxy(1,4); printf("angle   (deg):");
	gotoxy(1,5); printf("position (mm):");

	last_position = 0.0;

	// read and do conversion
	while (!done) {

		// read and print sensor
		sensor = (float) read_ad();
		gotoxy(16,3); printf("%5f  ", sensor);

		// calculate and print angle
		angle = a*sensor*sensor*sensor + b*sensor*sensor + c*sensor + d;
		gotoxy(16,4); printf("%3.2f  ", angle + 90.0);

		// calculate and print position
		position = 0.9*(angle-180.0)*DEG2RAD*LX + 0.1*last_position;
		gotoxy(16,5); printf("%3.2f  ", position);

		// erase last ball
		setfillstyle(SOLID_FILL, BLACK);
		fillellipse((int)(last_position*2) + 320, 300, 5, 5);

		// draw new ball
		setfillstyle(SOLID_FILL, WHITE);
		fillellipse((int)(position*2) + 320, 300, 5, 5);

		last_position = position;
		if (kbhit()) done = HI;
	}

	getch();
	closegraph();
}

/*-------------------- simple_cal --------------------------*
 *  Simple linear calibration routine                       *
 *----------------------------------------------------------*/
void simple_cal(void)
{
	int error = LO;
	char grpnum[4], filename[15];
	FILE *fp;
	long int sensor;
	char angle[10];

	// read only one ad channel
	scan_channels(0,0);

	// Prompt for group number
	clrscr();
	printf("Enter your group number (2 characters max): ");
	gets(grpnum);
	printf("\n");

	// Open file
	sprintf(filename, "data%s.dat", grpnum);
	fp = fopen(filename, "w");
	if (fp == NULL) {
	printf("Error opening file.");
		error = HI;
	} else printf ("Your data file will be called %s. \n\n", filename);

	if (!error) {
		printf("You are now ready to start taking data\n");
		printf("Hit any key to begin");
		while (!kbhit()) ;
		getch();
		clrscr();

		// Taking data
		textcolor(WHITE); gotoxy(1,2);
		printf("For each measurement, follow the instructions at the bottom of the screen\n");

		// table labels
		textcolor(GREEN);
		gotoxy( 1,10); cprintf("#");
		gotoxy( 5,10); cprintf("Reading from sensor");
		gotoxy(30,10); cprintf("Angle (deg)");

		// measurement 1
		textcolor(CYAN);
		gotoxy(1,20);
		cprintf("Measurement #1:\n");
		gotoxy(1,21);
		cprintf("Hold the sector in the center (theta = 0), then hit a key to store value\n");
		gotoxy(1,22);
		textcolor(WHITE);
		gotoxy(1,11); printf("1");
		while(!kbhit()) {
			sensor = read_ad();             // read the a/d converter
			gotoxy(5,11); printf("%ld", sensor);
		}
		getch();
		gotoxy(30,11); printf("0.0");
		fprintf(fp, "1 \t %ld \t 0.0 \t\n", sensor);

		// measurement 2
		textcolor(CYAN);
		gotoxy(1,20);
		cprintf("Measurement #2:\n");
		gotoxy(1,21); clreol(); gotoxy(1,21);
		cprintf("Tilt the sector clockwise until it rests against the stop then hit a key\n");
		gotoxy(1,23);
		textcolor(WHITE);
		gotoxy(1,12); cprintf("2");
		while(!kbhit()) {
			sensor = read_ad();             // read the a/d converter
			gotoxy(5,12); cprintf("%ld", sensor);
		}
		getch();
		textcolor(CYAN);
		gotoxy(1,22);
		cprintf("Measure the angle in degrees, type in the value, and hit enter\n");
		gotoxy(30,12);
		gets(angle);
		textcolor(WHITE);
		gotoxy(30,12); cprintf("%s", angle);
		fprintf(fp, "2 \t %ld \t %s \t\n", sensor, angle);
		gotoxy(1,22); clreol();
		gotoxy(1,23); clreol();

		// measurement 3
		textcolor(CYAN);
		gotoxy(1,20);
		cprintf("Measurement #3:\n");
		gotoxy(1,21); clreol(); gotoxy(1,21);
		cprintf("Tilt the sector counterclockwise until it rests against the stop then hit a key\n");
		gotoxy(1,23);
		textcolor(WHITE);
		gotoxy(1,13); printf("3");
		while(!kbhit()) {
			sensor = read_ad();             // read the a/d converter
		gotoxy(5,13); printf("%ld", sensor);
		}
		getch();
		textcolor(CYAN);
		gotoxy(1,22);
		cprintf("Measure the angle in degrees, type in the value, and hit enter\n");
		gotoxy(30,13);
		gets(angle);
		textcolor(WHITE);
		gotoxy(30,13); cprintf("%s", angle);
		fprintf(fp, "3 \t %ld \t %s \t\n", sensor, angle);

		// end
		textcolor(CYAN);
		gotoxy(1,20);
		cprintf("Measurements Complete. Hit any key to return to main menu.\n");
		gotoxy(1,21); clreol();
		gotoxy(1,22); clreol();
		gotoxy(1,23); clreol();
		while(!kbhit());
		getch();

	fclose(fp);
	}
}

/*------------------ test_simple_cal -----------------------*
 *  Test simple calibration                                 *
 *----------------------------------------------------------*/
void test_simple_cal(void)
{
	int error = LO, done = LO;
	char grpnum[4], filename[15], string[40];
	FILE *fp;
	int index[3];
	long int sensor, sensor_data[3];
	float angle, angle_data[3], last_angle;
	float slope, offset;

	// read only one ad channel
	scan_channels(0,0);

	// Prompt for group number
	clrscr();
	printf("Testing Calibration...\n");
	printf("Enter your group number (2 characters max): ");
	gets(grpnum);
	printf("\n");

	// Open file
	sprintf(filename, "data%s.dat", grpnum);
	fp = fopen(filename, "r");
	if (fp == NULL) {
	printf("Error opening file.");
		error = HI;
	} else printf ("Opening data file %s. \n\n", filename);

	if (!error) {
		init_graphics();

		// load in array from data file
		fgets(string,40,fp);
		sscanf(string, "%d \t %ld \t %f\n", &index[0], &sensor_data[0], &angle_data[0]);
		fgets(string,40,fp);
		sscanf(string, "%d \t %ld \t %f\n", &index[1], &sensor_data[1], &angle_data[1]);
		fgets(string,40,fp);
		sscanf(string, "%d \t %ld \t %f\n", &index[2], &sensor_data[2], &angle_data[2]);

		printf("%d \t %ld \t %f \t\n", index[0], sensor_data[0], angle_data[0]);
		printf("%d \t %ld \t %f \t\n", index[1], sensor_data[1], angle_data[1]);
		printf("%d \t %ld \t %f \t\n", index[2], sensor_data[2], angle_data[2]);

		slope = ((float)(sensor_data[1]-sensor_data[2]))/(angle_data[1]-angle_data[2]);
		offset = (float) sensor_data[0];

		printf("\nSlope in bits per degree: %f\n", slope);
		printf("Offset from zero in bits: %ld\n", sensor_data[0]);

		printf("\nsensor:   angle:");
	}

	last_angle = 0.0;

	// read and do conversion
	while (!done && !error) {

		// read and print sensor
		sensor = read_ad();
		gotoxy(1,9); printf("%ld     ", sensor);

		// calculate and print angle
		angle = ((float)sensor - offset)/slope;
		gotoxy(11,9); printf("%3.3f     ", angle);

		// erase last ball
		setfillstyle(SOLID_FILL, BLACK);
		fillellipse((int)(last_angle*3) + 320, 400, 5, 5);

		// draw new ball
		setfillstyle(SOLID_FILL, WHITE);
		fillellipse((int)(angle*3) + 320, 400, 5, 5);

		last_angle = angle;
		if (kbhit()) done = HI;
	}

	getch();
	closegraph();
	fclose(fp);
}
